@@ -391,7 +391,7 @@ class AstrometryNetClass(BaseQuery):
 
             print("Determining background stats", flush=True)
             mean, median, std = sigma_clipped_stats(data, sigma=3.0,
-                                                    iters=5)
+                                                    maxiters=5)
             daofind = DAOStarFinder(fwhm=fwhm,
                                     threshold=detect_threshold * std)
             print("Finding sources", flush=True)
