@@ -20,9 +20,6 @@ class Conf(_config.ConfigNamespace):
     ssoserver = _config.ConfigItem(
         'https://ssoportal.stsci.edu',
         'MAST SSO Portal server.')
-    catalogsserver = _config.ConfigItem(
-        'https://catalogs.mast.stsci.edu',
-        'Name of the MAST Catalogs server.')
     timeout = _config.ConfigItem(
         600,
         'Time limit for requests from the STScI server.')
@@ -33,13 +30,15 @@ class Conf(_config.ConfigNamespace):
 
 conf = Conf()
 
-
-from .core import Observations, ObservationsClass, Catalogs, CatalogsClass, Mast, MastClass
-from .tesscut import TesscutClass, Tesscut
+from .cutouts import TesscutClass, Tesscut
+from .observations import Observations, ObservationsClass, MastClass, Mast
+from .collections import Catalogs, CatalogsClass
+from .core import MastQueryWithLogin
+from . import utils
 
 __all__ = ['Observations', 'ObservationsClass',
            'Catalogs', 'CatalogsClass',
            'Mast', 'MastClass',
            'Tesscut', 'TesscutClass',
-           'Conf', 'conf',
+           'Conf', 'conf', 'utils',
            ]
