@@ -12,9 +12,9 @@ class MockResponse(object):
     A mocked/non-remote version of `astroquery.query.AstroResponse`
     """
 
-    def __init__(self, content=None, url=None, headers={},
-                 content_type=None, stream=False, auth=None, status_code=200,
-                 verify=True):
+    def __init__(self, content=None, url=None, headers={}, content_type=None,
+                 stream=False, auth=None, status_code=200, verify=True,
+                 allow_redirects=True, json=None):
         assert content is None or hasattr(content, 'decode')
         self.content = content
         self.raw = content
